cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      module systems
c      两体系统的变量
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       implicit none
       real*8 :: z1,z2 ! 两体系统的电荷数
       real*8 :: mass1, mass2 ! 两体系统的质量数
       real*8 :: be ! 结合能 be < 0
      end module
c-----------------------------------------------------------------------




      program np_bound_r_space
      use mesh
      use systems
      use gauss
      implicit none

      ! 定义格点信息
      hcm=0.01 ! fm
      irmatch=2000
      nr=40
      allocate(rr(1:nr),rrw(1:nr))
      call gauleg(nr,0.0d0,hcm*irmatch,rr,rrw) ! 初始化高斯积分

      !定义系统信息
      z1=1.0
      z2=0.0
      mass1=1.0078
      mass2=1.0086
      be=-2.224 ! MeV

      call npbound()



      end program




ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine npbound()
c      使用R矩阵方法求解薛定谔方程确定束缚态能量
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       use mesh
       use systems
       use constants
       use lagrange_mesh_single_channel
       implicit none
       integer :: ir ! 格点信息
       real*8 :: mu,ecm ! 折合质量， 与质心系能量
       real*8 :: norm !  归一化系数
       real*8,dimension(0:0) :: WK,WKD
       real*8 :: knp ! wave number
       real*8,dimension(0:irmatch) :: vpot ! np相互作用势
       real*8,dimension(1:nr) ::  phi ! np波函数
       real*8 :: r
       real*8 :: V0 ! 初始势阱深度
       real*8 :: gausspot ! 高斯势函数
       integer :: l,nodes


       phi=0.0d0
       vpot=0.0d0
       l=0
       nodes=1
       ! 定义势能
       V0=-72.167
       do ir=0, irmatch
         r=ir*hcm
         vpot(ir) = gausspot(r,v0,0.0d0,1.484d0)  ! 注意此处没有加库伦力
         write(8,*) r, vpot(ir)
       end do

       call flush(8)
       mu=amu*(mass2*mass1)/(mass1+mass2)
       rmax=irmatch*hcm

       call initial_lagrange_func(rmax)
       call T_operator(mu)
       call centrifugal_barrier (l,mu)
       call lagrange_V(Vpot)
       call rmatrix_bound(nodes,mu,z1*z2,l,be,phi)




      end subroutine npbound
c-----------------------------------------------------------------------

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine rmatrix_bound(nnode,mu,z12,l,E1,phi)
!     R-matrix method for bound state calculation
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       use precision
       use constants
       use whittaker
       use lagrange_mesh_single_channel
       use systems
       implicit none
       real*8 ::  tolerance
       real*8 :: E0,E1
       real*8 :: k, eta, mu ,z12
       integer :: IE,l,nnode
       real*8,dimension(0:l) :: WK,WKD
       real*8 :: bc
       real*8,dimension(1:nr, 1:nr) :: B_operator,A
       real*8,dimension(1:nr) :: wf, phi
       integer :: ir
       real*8 :: norm

       B_operator=0.0_dpreal
       A=0.0_dpreal
       tolerance=0.001
       E0=-99.0_dpreal
       E1=-0.000001_dpreal
       do while(abs(E1-E0)>tolerance)
         k=sqrt(-2*mu*E1/(hbarc**2))
         eta=z12*e2*mu/hbarc/hbarc/k
         IE=0
         call WHIT(eta,rmax,k,E1,l,WK,WKD,IE)
         bc= k*rmax*WKD(l)/WK(l)
         call Bloch_bound(mu,Bc,B_operator)
         A= B_operator+ kinetic_matrix + l_barrier_matrix + V_matrix
         E0=E1
         call eigenvalue_Rmat(nnode,A,E1,wf)
       end do

       write(*,*) "Binding energy is",E1


       do ir=1, nr
        phi(ir)= lagrange_func(ir)*wf(ir)
       end do

      !  normalized the wave function
       norm=0.0_dpreal
       do ir=1, nr
          norm=norm+ phi(ir) * phi(ir) * rrw(ir)
       end do

       phi= phi / norm

       if(phi(1)<0) phi=-phi
       write(7,*) "& "

       do ir=1, nr
          write(7,*) rr(ir), phi(ir)
       end do



      end subroutine
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine eigenvalue_Rmat(nnode,Ain,E,wf)
      use mesh
      implicit none
      real*8 :: E
      integer :: n,nnode
      integer :: info,lwork,loc(1)
      real*8, dimension(1:nr) :: wr,wi,wf
      real*8, dimension(1:4*nr) :: work
      real*8,dimension(1:nr,1:nr) :: vl,vr
      real*8,dimension(1:nr,1:nr) :: A, Ain
      integer :: ir

      A=Ain
! prepare calling the LAPACK routine DGEEV
      lwork=4*nr !  the value is taken from the documentation of DGEEV
! the routine will destroy the original array A
! and return the real parts of the eigenvalues in WR and the imaginary parts in WI
! VR(:,i) contain the eigenstate (=wave function) of the right eigenvector of eigenvalue i
! VL(:,i) contain the eigenstate of the left eigenvector of eigenvalue i

      call DGEEV('N','V',NR,A,NR,WR,WI,VL,NR,VR,NR,WORK,LWORK,INFO)


      if (info/=0) then
         write(*,*) "problem happened when calling DGEEV"
         stop
      end if

! locate maximal eigenvector
! here assume that this eigenvector is real
! then return this eigenvalue in ETA
! and the eigenvector in the global variable WF (which should be of dimension NR)

      do n=1,nnode-1
       loc=MINLOC(wr) ! return value is type of array, must defined with array type
       wr(loc(1))=0
      end do
       loc=MINLOC(wr)


       E=wr(loc(1))
       wf=vr(:,loc(1))

      end subroutine


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c *** Gaussian
      function gausspot(r,v0,r0,a)
      implicit none
       real*8 r,v0,r0,gausspot,a
         if (a.gt.1e-6) then
           gausspot=V0*exp(-(r-r0)**2/a**2)
             else
               write(*,*)'a too small in gausspot!'
               stop
         endif
         return
      end function
